import oscP5.*;
import netP5.*;
  
OscP5 oscP5;

int ScreenWidth = 10;
int ScreenHeight = 10;
int FrameRate = 120;

int inputPort;
NetAddress outputAddress;

LinkedList uniqueMessages = null;
LinkedList filterMessages = null;

// ------------------------------------------------------------------------------------------------
void loadConfiguration(String filename){
  println("OSCFilterForwarder: loading file " + filename);
  String[] data = loadStrings(filename);
  
  if(data.length<2){
    println("OSCFilterForwarder: ERROR - the config file must have at least 3 lines");
    println("OSCFilterForwarder: listening port");
    println("OSCFilterForwarder: output ip address");
    println("OSCFilterForwarder: output port");
    exit();
  }
  
  inputPort = Integer.parseInt(data[0]);
  println("OSCFilterForwarder: input port " + inputPort);
  
  outputAddress = new NetAddress(data[1], parseInt(data[2]));
  println("OSCFilterForwarder: output address " + data[1]);
  println("OSCFilterForwarder: output port " + data[2]);
  
  for(int i=3; i<data.length; i++){
    data[i] = data[i].toLowerCase();
    if(i == 3){
      filterMessages = new LinkedList(data[i]);
    } else {
      filterMessages.insertAfter(data[i]);
    }
  }
  println("OSCFilterForwarder: blocking all other OSC message(s)");        
}

// ------------------------------------------------------------------------------------------------
boolean shouldMessageBeForwarded(String addrPattern){
  if(filterMessages == null){
    return false;
  }
  
  String pat = addrPattern.toLowerCase();  
  for(LinkedList c = filterMessages; c!=null; c = c.next){   
    int comparisonResult = pat.indexOf(c.getValueAsString()); 
    if(comparisonResult != -1){
      return true;
    }
  }
  
  return false;  
}

// ------------------------------------------------------------------------------------------------
void processUniqueMessages(String addrPattern){
  String pat = addrPattern.toLowerCase();

  if(uniqueMessages == null){
    println("OSCFilterForwarder: received a new type of message " + addrPattern);
    uniqueMessages = new LinkedList(pat);
    return;
  }

  for(LinkedList c = uniqueMessages; c!=null; c = c.next){
    if(c.getValueAsString().toLowerCase().indexOf(pat) != -1){
      return;
    }
  }
  
  println("OSCFilterForwarder: received a new type of message " + addrPattern);
  uniqueMessages.insertAfter(addrPattern);
}

// ------------------------------------------------------------------------------------------------
void saveUniqueMessages(String filename){
  int len = 0;
  if(uniqueMessages != null){
    len = uniqueMessages.length();
  }
  
  println("OSCFilterForwarder: saving " + len + " unique messages to file ./data/" + filename);
  
  if(len == 0){
    return;
  }
  
  String[] s = new String[len];
  int i=0;
  for(LinkedList c = uniqueMessages; c!=null; c = c.next){
    s[i]=c.getValueAsString();
    i+=1;
  }
  
  saveStrings("data/"+filename, s);
}

// ------------------------------------------------------------------------------------------------
void setup() {
  println("------------------------------------------------------------------------------------------");
  println("OSCFilterForwarder: press 'f' or 'F' to list the filtered messages (which will be forwarded)");
  println("OSCFilterForwarder: press 'l' or 'L' to list the unique messages so far");
  println("OSCFilterForwarder: press 's' or 'S' to save the list of unique messages so far");
  println("------------------------------------------------------------------------------------------");
  loadConfiguration("config.txt");
  println("------------------------------------------------------------------------------------------");
  println("\n");
  
  size(ScreenWidth, ScreenHeight);
  frameRate(FrameRate);
  
  //set this to the receiving port
  OscProperties myProperties = new OscProperties();
  myProperties.setDatagramSize(30000); 
  myProperties.setListeningPort(inputPort); 
  oscP5 = new OscP5(this, myProperties);
  // oscP5 = new OscP5(this, inputPort);
}

// ------------------------------------------------------------------------------------------------
void draw() {
  background(0); 
  
//  if((frameCount % FrameRate) == 0){
//    println("OSCFilterForwarde: framerate = " + frameRate);
//  }
}

// ------------------------------------------------------------------------------------------------
void keyPressed(){
  if(key == 'l' || key == 'L'){
     println("OSCFilterForwarder: listing unique messages so far");
     if(uniqueMessages!=null){
      uniqueMessages.debugPrint();
    }
  }

 if(key == 'f' || key == 'F'){
    println("OSCFilterForwarder: listing the filtered messages (which will be forwarded)");
    if(filterMessages!=null){
        filterMessages.debugPrint();
    }
  }

 if(key == 's' || key == 'S'){
   saveUniqueMessages("uniqueMessages.txt");
  }
}


// ------------------------------------------------------------------------------------------------
void oscEvent(OscMessage msg) { 
  
  if(shouldMessageBeForwarded(msg.addrPattern())){
    oscP5.send(msg, outputAddress);
  }
  
  processUniqueMessages(msg.addrPattern());  
}
